@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto & The Sylpheed Claws Team
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto & The Sylpheed Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +35,9 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
-#include <sys/wait.h>
+#if HAVE_SYS_WAIT_H
+#  include <sys/wait.h>
+#endif
 #include <signal.h>
 #include <unistd.h>
 
@@ -579,6 +581,7 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 			gtk_text_buffer_get_iter_at_offset
 				(textbuf, &start_iter, body_pos);
 			gtk_text_buffer_get_end_iter(textbuf, &end_iter);
+			gtk_text_buffer_place_cursor(textbuf, &start_iter);
 		}
 		msg_str = gtk_text_buffer_get_text
 			(textbuf, &start_iter, &end_iter, FALSE);
@@ -688,6 +691,7 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 			     Children *children)
 {
+#ifdef G_OS_UNIX
 	gint chld_in[2], chld_out[2], chld_err[2], chld_status[2];
 	gchar *cmdline[4];
 	pid_t pid, gch_pid;
@@ -845,20 +849,25 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 	}
 
 	return child_info;
+#else
+	return NULL;
+#endif /* G_OS_UNIX */
 }
 
 static void kill_children_cb(GtkWidget *widget, gpointer data)
 {
+#ifdef G_OS_UNIX
 	GSList *cur;
 	Children *children = (Children *) data;
 	ChildInfo *child_info;
 
 	for (cur = children->list; cur; cur = cur->next) {
 		child_info = (ChildInfo *)(cur->data);
-		debug_print("Killing child group id %d\n", child_info->pid);
-		if (child_info->pid && kill(-child_info->pid, SIGTERM) < 0)
+		debug_print("Killing child: %d\n", child_info->pid);
+		if (child_info->pid && kill(child_info->pid, SIGTERM) < 0)
 			perror("kill");
 	}
+#endif /* G_OS_UNIX */
 }
 
 static gint wait_for_children(Children *children)
@@ -970,6 +979,14 @@ static void free_children(Children *children)
 	g_free(children->action);
 	while (children->list != NULL) {
 		child_info = (ChildInfo *)children->list->data;
+#ifdef G_OS_UNIX
+		if (child_info->pid != 0) {
+			if (waitpid(child_info->pid, NULL, 0)
+			    != child_info->pid) {
+				perror("waitpid (free_children)");
+			}
+		}
+#endif
 		g_free(child_info->cmd);
 		g_string_free(child_info->output, TRUE);
 		children->list = g_slist_remove(children->list, child_info);
@@ -1156,7 +1173,9 @@ static void catch_status(gpointer data, gint source, GdkInputCondition cond)
 	c = read(source, &buf, 1);
 	debug_print("Child returned %c\n", buf);
 
-	waitpid(-child_info->pid, NULL, 0);
+#ifdef G_OS_UNIX
+	waitpid(child_info->pid, NULL, 0);
+#endif
 	childinfo_close_pipes(child_info);
 	child_info->pid = 0;
 
