@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <gdk/gdkkeysyms.h>
 
 #include "alertpanel.h"
+#include "mainwindow.h"
 #include "manage_window.h"
 #include "utils.h"
 #include "gtkutils.h"
@@ -64,6 +65,9 @@ static gint alertpanel_deleted		(GtkWidget		*widget,
 static gboolean alertpanel_close	(GtkWidget		*widget,
 					 GdkEventAny		*event,
 					 gpointer		 data);
+static gint alertpanel_focus_out	(GtkWidget		*widget,
+					 GdkEventFocus		*event,
+					 gpointer		 data);
 
 AlertValue alertpanel_full(const gchar *title, const gchar *message,
 			   AlertType type, AlertValue default_value,
@@ -166,11 +170,24 @@ void alertpanel_error(const gchar *format, ...)
 
 static void alertpanel_show(void)
 {
-	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
-	manage_window_set_transient(GTK_WINDOW(dialog));
+	gint x, y, w, h, sx, sy;
 	value = G_ALERTWAIT;
 
 	inc_lock();
+
+	sx = gdk_screen_width();
+	sy = gdk_screen_height();
+	gdk_window_get_origin(dialog->window, &x, &y);
+	w = dialog->allocation.width;
+	h = dialog->allocation.height;
+	if (x < 0 || y < 0 || x + w > sx || y + h > sy) {
+		debug_print("sx, sy,  x, y,  w, h = %d, %d,  %d, %d,  %d, %d\n",
+			    sx, sy, x, y, w, h);
+		debug_print("alert dialog position out of range\n");
+		gtk_window_set_position(GTK_WINDOW(dialog),
+					GTK_WIN_POS_CENTER_ALWAYS);
+	}
+
 	while ((value & G_ALERT_VALUE_MASK) == G_ALERTWAIT)
 		gtk_main_iteration();
 
@@ -208,7 +225,10 @@ static void alertpanel_create(const gchar *title,
 	dialog = gtk_dialog_new();
 	gtk_window_set_title(GTK_WINDOW(dialog), title);
 	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, FALSE);
-	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
+	gtk_window_set_position(GTK_WINDOW(dialog),
+				GTK_WIN_POS_CENTER_ON_PARENT);
+	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
+	manage_window_set_transient(GTK_WINDOW(dialog));
 	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
 	g_signal_connect(G_OBJECT(dialog), "delete_event",
 			 G_CALLBACK(alertpanel_deleted),
@@ -216,6 +236,8 @@ static void alertpanel_create(const gchar *title,
 	g_signal_connect(G_OBJECT(dialog), "key_press_event",
 			 G_CALLBACK(alertpanel_close),
 			 (gpointer)G_ALERTCANCEL);
+	g_signal_connect(G_OBJECT(dialog), "focus_out_event",
+			 G_CALLBACK(alertpanel_focus_out), NULL);
 
 	/* for title icon, label and message */
 	hbox = gtk_hbox_new(FALSE, 12);
@@ -367,3 +389,13 @@ static gboolean alertpanel_close(GtkWidget *widget, GdkEventAny *event,
 	value = (value & ~G_ALERT_VALUE_MASK) | (AlertValue)data;
 	return FALSE;
 }
+
+
+static gint alertpanel_focus_out(GtkWidget *widget, GdkEventFocus *event,
+				 gpointer data)
+{
+#ifdef G_OS_WIN32
+	gtk_window_present(GTK_WINDOW(widget));
+#endif
+	return FALSE;
+}
