@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,13 +33,6 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
-#include <time.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/wait.h>
-#include <signal.h>
-#include <errno.h>
 
 #include "main.h"
 #include "inc.h"
@@ -61,8 +54,8 @@
 #include "manage_window.h"
 #include "stock_pixmap.h"
 #include "progressdialog.h"
-#include "inputdialog.h"
 #include "alertpanel.h"
+#include "trayicon.h"
 #include "filter.h"
 #include "folder.h"
 
@@ -78,6 +71,9 @@ static GdkPixbuf *ok_pixbuf;
 
 static void inc_finished		(MainWindow		*mainwin,
 					 gint			 new_messages);
+
+static gint inc_remote_account_mail	(MainWindow		*mainwin,
+					 PrefsAccount		*account);
 static gint inc_account_mail_real	(MainWindow		*mainwin,
 					 PrefsAccount		*account);
 
@@ -156,7 +152,17 @@ static void inc_finished(MainWindow *mainwin, gint new_messages)
 	debug_print("inc_finished(): %d new message(s)\n", new_messages);
 
 	if (prefs_common.scan_all_after_inc)
-		folderview_check_new(NULL);
+		new_messages += folderview_check_new(NULL);
+
+	if (new_messages > 0) {
+		gchar buf[1024];
+
+		g_snprintf(buf, sizeof(buf), _("Sylpheed: %d new messages"),
+			   new_messages);
+		trayicon_set_tooltip(buf);
+	} else {
+		trayicon_set_tooltip(NULL);
+	}
 
 	if (new_messages <= 0 && !prefs_common.scan_all_after_inc) return;
 
@@ -176,10 +182,9 @@ static void inc_finished(MainWindow *mainwin, gint new_messages)
 	    prefs_common.enable_newmsg_notify &&
 	    prefs_common.newmsg_notify_cmd) {
 		gchar buf[1024];
-		gchar *p;
 
-		if ((p = strchr(prefs_common.newmsg_notify_cmd, '%')) &&
-		    *(p + 1) == 'd' && !strchr(p + 2, '%'))
+		if (str_find_format_times
+			(prefs_common.newmsg_notify_cmd, 'd') == 1)
 			g_snprintf(buf, sizeof(buf),
 				   prefs_common.newmsg_notify_cmd,
 				   new_messages);
@@ -228,20 +233,37 @@ void inc_mail(MainWindow *mainwin)
 	inc_autocheck_timer_set();
 }
 
+static gint inc_remote_account_mail(MainWindow *mainwin, PrefsAccount *account)
+{
+	FolderItem *item = mainwin->summaryview->folder_item;
+	gint new_msgs;
+
+	g_return_val_if_fail(account->folder != NULL, 0);
+
+	if (account->protocol == A_IMAP4 && account->imap_check_inbox_only) {
+		FolderItem *inbox = FOLDER(account->folder)->inbox;
+
+		new_msgs = folderview_check_new_item(inbox);
+		if (!prefs_common.scan_all_after_inc && item != NULL &&
+		    inbox == item)
+			folderview_update_item(item, TRUE);
+	} else {
+		new_msgs = folderview_check_new(FOLDER(account->folder));
+		if (!prefs_common.scan_all_after_inc && item != NULL &&
+		    FOLDER(account->folder) == item->folder)
+			folderview_update_item(item, TRUE);
+	}
+
+	return new_msgs;
+}
+
 static gint inc_account_mail_real(MainWindow *mainwin, PrefsAccount *account)
 {
 	IncProgressDialog *inc_dialog;
 	IncSession *session;
 
-	if (account->protocol == A_IMAP4 || account->protocol == A_NNTP) {
-		FolderItem *item = mainwin->summaryview->folder_item;
-
-		folderview_check_new(FOLDER(account->folder));
-		if (!prefs_common.scan_all_after_inc && item != NULL &&
-		    FOLDER(account->folder) == item->folder)
-			folderview_update_item(item, TRUE);
-		return 1;
-	}
+	if (account->protocol == A_IMAP4 || account->protocol == A_NNTP)
+		return inc_remote_account_mail(mainwin, account);
 
 	session = inc_session_new(account);
 	if (!session) return 0;
@@ -304,14 +326,8 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 	for (list = account_get_list(); list != NULL; list = list->next) {
 		PrefsAccount *account = list->data;
 		if ((account->protocol == A_IMAP4 ||
-		     account->protocol == A_NNTP) && account->recv_at_getall) {
-			FolderItem *item = mainwin->summaryview->folder_item;
-
-			folderview_check_new(FOLDER(account->folder));
-			if (!prefs_common.scan_all_after_inc && item != NULL &&
-			    FOLDER(account->folder) == item->folder)
-				folderview_update_item(item, TRUE);
-		}
+		     account->protocol == A_NNTP) && account->recv_at_getall)
+			new_msgs += inc_remote_account_mail(mainwin, account);
 	}
 
 	/* check POP3 accounts */
@@ -374,8 +390,8 @@ static IncProgressDialog *inc_progress_dialog_create(gboolean autocheck)
 	}
 
 	dialog->dialog = progress;
-	gettimeofday(&dialog->progress_tv, NULL);
-	gettimeofday(&dialog->folder_tv, NULL);
+	g_get_current_time(&dialog->progress_tv);
+	g_get_current_time(&dialog->folder_tv);
 	dialog->queue_list = NULL;
 	dialog->cur_row = 0;
 
@@ -492,7 +508,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 					(inc_dialog->dialog->window,
 					 NULL, NULL);
 
-			pass = input_dialog_query_password
+			pass = input_query_password
 				(pop3_session->ac_prefs->recv_server,
 				 pop3_session->user);
 
@@ -698,6 +714,8 @@ static IncState inc_pop3_session_do(IncSession *session)
 	session_set_timeout(SESSION(pop3_session),
 			    prefs_common.io_timeout_secs * 1000);
 
+	GTK_EVENTS_FLUSH();
+
 	if (session_connect(SESSION(pop3_session), server, port) < 0) {
 		log_warning(_("Can't connect to POP3 server: %s:%d\n"),
 			    server, port);
@@ -826,8 +844,8 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 	gchar buf[MSGBUFSIZE];
 	Pop3Session *pop3_session = POP3_SESSION(inc_session->session);
 	gchar *total_size_str;
-	gint cur_total;
-	gint total;
+	gint64 cur_total;
+	gint64 total;
 
 	if (!pop3_session->new_msg_exist) return;
 
@@ -887,17 +905,17 @@ static void inc_update_folderview(IncProgressDialog *inc_dialog,
 static void inc_progress_dialog_update_periodic(IncProgressDialog *inc_dialog,
 						IncSession *inc_session)
 {
-	struct timeval tv_cur;
-	struct timeval tv_result;
+	GTimeVal tv_cur;
+	GTimeVal tv_result;
 	gint msec;
 
-	gettimeofday(&tv_cur, NULL);
+	g_get_current_time(&tv_cur);
 
 	tv_result.tv_sec = tv_cur.tv_sec - inc_dialog->progress_tv.tv_sec;
 	tv_result.tv_usec = tv_cur.tv_usec - inc_dialog->progress_tv.tv_usec;
 	if (tv_result.tv_usec < 0) {
 		tv_result.tv_sec--;
-		tv_result.tv_usec += 1000000;
+		tv_result.tv_usec += G_USEC_PER_SEC;
 	}
 
 	msec = tv_result.tv_sec * 1000 + tv_result.tv_usec / 1000;
@@ -911,17 +929,17 @@ static void inc_progress_dialog_update_periodic(IncProgressDialog *inc_dialog,
 static void inc_update_folderview_periodic(IncProgressDialog *inc_dialog,
 					   IncSession *inc_session)
 {
-	struct timeval tv_cur;
-	struct timeval tv_result;
+	GTimeVal tv_cur;
+	GTimeVal tv_result;
 	gint msec;
 
-	gettimeofday(&tv_cur, NULL);
+	g_get_current_time(&tv_cur);
 
 	tv_result.tv_sec = tv_cur.tv_sec - inc_dialog->folder_tv.tv_sec;
 	tv_result.tv_usec = tv_cur.tv_usec - inc_dialog->folder_tv.tv_usec;
 	if (tv_result.tv_usec < 0) {
 		tv_result.tv_sec--;
-		tv_result.tv_usec += 1000000;
+		tv_result.tv_usec += G_USEC_PER_SEC;
 	}
 
 	msec = tv_result.tv_sec * 1000 + tv_result.tv_usec / 1000;
@@ -938,7 +956,7 @@ static gint inc_recv_data_progressive(Session *session, guint cur_len,
 	IncSession *inc_session = (IncSession *)data;
 	Pop3Session *pop3_session = POP3_SESSION(session);
 	IncProgressDialog *inc_dialog;
-	gint cur_total;
+	gint64 cur_total;
 
 	g_return_val_if_fail(inc_session != NULL, -1);
 
@@ -1038,11 +1056,18 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	fltinfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
 	fltinfo->flags.tmp_flags = MSG_RECEIVED;
 
-	if (session->ac_prefs->filter_on_recv)
+	if (prefs_common.enable_junk &&
+	    prefs_common.filter_junk_on_recv &&
+	    prefs_common.filter_junk_before)
+		filter_apply(prefs_common.junk_fltlist, file, fltinfo);
+
+	if (!fltinfo->drop_done && session->ac_prefs->filter_on_recv)
 		filter_apply(prefs_common.fltlist, file, fltinfo);
+
 	if (!fltinfo->drop_done) {
 		if (prefs_common.enable_junk &&
-		    prefs_common.filter_junk_on_recv)
+		    prefs_common.filter_junk_on_recv &&
+		    !prefs_common.filter_junk_before)
 			filter_apply(prefs_common.junk_fltlist, file, fltinfo);
 	}
 
@@ -1060,13 +1085,9 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 
 		val = GPOINTER_TO_INT(g_hash_table_lookup
 				      (inc_session->folder_table, drop_folder));
-		if (val == 0) {
-			/* force updating */
-			if (FOLDER_IS_LOCAL(drop_folder->folder))
-				drop_folder->mtime = 0;
-			g_hash_table_insert(inc_session->folder_table, drop_folder,
-					    GINT_TO_POINTER(1));
-		}
+		if (val == 0)
+			g_hash_table_insert(inc_session->folder_table,
+					    drop_folder, GINT_TO_POINTER(1));
 		g_hash_table_insert(inc_session->tmp_folder_table, drop_folder,
 				    GINT_TO_POINTER(1));
 	}
@@ -1276,7 +1297,7 @@ static gint get_spool(FolderItem *dest, const gchar *mbox)
 		folder_table = g_hash_table_new(NULL, NULL);
 	msgs = proc_mbox(dest, tmp_mbox, folder_table);
 
-	unlink(tmp_mbox);
+	g_unlink(tmp_mbox);
 	if (msgs >= 0) empty_mbox(mbox);
 	unlock_mbox(mbox, lockfd, LOCK_FLOCK);
 
