@@ -32,6 +32,8 @@
 #include "addrcache.h"
 
 #include "base64.h"
+#include "codeconv.h"
+#include "utils.h"
 
 /*
 * Create new object.
@@ -192,7 +194,7 @@ void ldif_print_file( LdifFile *ldifFile, FILE *stream ) {
 static gint ldif_open_file( LdifFile* ldifFile ) {
 	/* printf( "Opening file\n" ); */
 	if( ldifFile->path ) {
-		ldifFile->file = fopen( ldifFile->path, "rb" );
+		ldifFile->file = g_fopen( ldifFile->path, "rb" );
 		if( ! ldifFile->file ) {
 			/* printf( "can't open %s\n", ldifFile->path ); */
 			ldifFile->retVal = MGU_OPEN_FILE;
@@ -227,30 +229,14 @@ static void ldif_close_file( LdifFile *ldifFile ) {
 */
 static gchar *ldif_get_line( LdifFile *ldifFile ) {
 	gchar buf[ LDIFBUFSIZE ];
-	gint ch;
-	gint i = 0;
 
 	if( feof( ldifFile->file ) )
 		return NULL;
 
-	while( i < LDIFBUFSIZE - 1 ) {
-		ch = fgetc( ldifFile->file );
-		if( ch == '\0' || ch == EOF ) {
-			if( i == 0 )
-				return NULL;
-			break;
-		}
-#if HAVE_DOSISH_SYSTEM
-#else
-		if( ch == '\r' )
-			continue;
-#endif
-		if( ch == '\n' )
-			break;
-		buf[i] = ch;
-		i++;
-	}
-	buf[i] = '\0';
+	if( fgets( buf, sizeof( buf ), ldifFile->file ) == NULL )
+		return NULL;
+
+	strretchomp( buf );
 
 	/* Return a copy of buffer */
 	return g_strdup( buf );
@@ -383,6 +369,7 @@ static void ldif_build_items( LdifFile *ldifFile, Ldif_ParsedRec *rec, AddressCa
 	gint iLen = 0, iLenT = 0;
 	ItemPerson *person;
 	ItemEMail *email;
+	gboolean familyFirst = FALSE;
 
 	nodeAddress = rec->listAddress;
 	if( nodeAddress == NULL ) return;
@@ -407,22 +394,45 @@ static void ldif_build_items( LdifFile *ldifFile, Ldif_ParsedRec *rec, AddressCa
 	if( rec->listLName ) {
 		lastName = rec->listLName->data;
 	}
-
-	if( firstName ) {
-		if( lastName ) {
-			fullName = g_strdup_printf( "%s %s", firstName, lastName );
-		}
-		else {
-			fullName = g_strdup_printf( "%s", firstName );
-		}
+	if( rec->listCName ) {
+		fullName = g_strdup((gchar *)rec->listCName->data);
 	}
-	else {
-		if( lastName ) {
-			fullName = g_strdup_printf( "%s", lastName );
+
+	familyFirst = conv_is_ja_locale();
+
+	if( fullName == NULL ) {
+		if( familyFirst ) {
+			if( lastName ) {
+				if( firstName ) {
+					fullName = g_strdup_printf( "%s %s", lastName, firstName );
+				}
+				else {
+					fullName = g_strdup_printf( "%s", lastName );
+				}
+			}
+			else {
+				if( firstName ) {
+					fullName = g_strdup_printf( "%s", firstName );
+				}
+			}
+		} else {
+			if( firstName ) {
+				if( lastName ) {
+					fullName = g_strdup_printf( "%s %s", firstName, lastName );
+				}
+				else {
+					fullName = g_strdup_printf( "%s", firstName );
+				}
+			}
+			else {
+				if( lastName ) {
+					fullName = g_strdup_printf( "%s", lastName );
+				}
+			}
 		}
 	}
 	if( fullName ) {
-		g_strchug( fullName ); g_strchomp( fullName );
+		g_strstrip( fullName );
 	}
 
 	if( rec->listNName ) {
@@ -510,7 +520,8 @@ static void ldif_add_value( Ldif_ParsedRec *rec, gchar *tagName, gchar *tagValue
 	else if( g_ascii_strcasecmp( nm, LDIF_TAG_LASTNAME ) == 0 ) {
 		rec->listLName = g_slist_append( rec->listLName, val );
 	}
-	else if( g_ascii_strcasecmp( nm, LDIF_TAG_NICKNAME ) == 0 ) {
+	else if( g_ascii_strcasecmp( nm, LDIF_TAG_NICKNAME ) == 0 ||
+	         g_ascii_strcasecmp( nm, LDIF_TAG_XNICKNAME ) == 0 ) {
 		rec->listNName = g_slist_append( rec->listNName, val );
 	}
 	else if( g_ascii_strcasecmp( nm, LDIF_TAG_EMAIL ) == 0 ) {
@@ -597,21 +608,43 @@ static void ldif_dump_b64( gchar *buf ) {
 	gchar outBuf[8192];
 	gint len;
 
-	printf( "base-64 : inbuf : %s\n", buf );
+	g_print( "base-64 : inbuf : %s\n", buf );
 	decoder = base64_decoder_new();
 	len = base64_decoder_decode( decoder, buf, outBuf );
 	if (len < 0) {
-		printf( "base-64 : Bad BASE64 content\n" );
+		g_print( "base-64 : Bad BASE64 content\n" );
 	}
 	else {
 		outBuf[len] = '\0';
-		printf( "base-64 : %d : %s\n\n", len, outBuf );
+		g_print( "base-64 : %d : %s\n\n", len, outBuf );
 	}
 	base64_decoder_free( decoder );
 	decoder = NULL;
 }
 #endif
 
+static gchar *ldif_conv_base64( gchar *buf ) {
+	gchar *outbuf;
+	gint len;
+
+	outbuf = g_malloc(strlen(buf) + 1);
+	len = base64_decode(outbuf, buf, -1);
+	outbuf[len] = '\0';
+	if (g_utf8_validate(outbuf, -1, NULL))
+		return outbuf;
+	else {
+		gchar *utf8str;
+
+		if (conv_is_ja_locale())
+			utf8str = conv_codeset_strdup(outbuf, NULL, NULL);
+		else
+			utf8str = conv_localetodisp(outbuf, NULL);
+
+		g_free(outbuf);
+		return utf8str;
+	}
+}
+
 /*
 * Read file data into address cache.
 * Note that one LDIF record identifies one entity uniquely with the
@@ -620,7 +653,7 @@ static void ldif_dump_b64( gchar *buf ) {
 */
 static void ldif_read_file( LdifFile *ldifFile, AddressCache *cache ) {
 	gchar *tagName = NULL, *tagValue = NULL;
-	gchar *lastTag = NULL, *fullValue = NULL;
+	gchar *lastTag = NULL;
 	GSList *listValue = NULL;
 	gboolean flagEOF = FALSE, flagEOR = FALSE;
 	gboolean flag64 = FALSE, last64 = FALSE;
@@ -639,7 +672,7 @@ static void ldif_read_file( LdifFile *ldifFile, AddressCache *cache ) {
 	fseek( ldifFile->file, 0L, SEEK_SET );
 
 	while( ! flagEOF ) {
-		gchar *line =  ldif_get_line( ldifFile );
+		gchar *line = ldif_get_line( ldifFile );
 
 		posCur = ftell( ldifFile->file );
 		if( ldifFile->cbProgress ) {
@@ -658,20 +691,24 @@ static void ldif_read_file( LdifFile *ldifFile, AddressCache *cache ) {
 		if( flagEOR ) {
 			/* EOR, Output address data */
 			if( lastTag ) {
+				gchar *fullValue;
+
 				/* Save record */
 				fullValue = mgu_list_coalesce( listValue );
 
 				/* Base-64 encoded data */
-				/*
 				if( last64 ) {
-					ldif_dump_b64( fullValue );
+					gchar *decValue;
+					decValue = ldif_conv_base64( fullValue );
+					g_free( fullValue );
+					fullValue = decValue;
 				}
-				*/
 
 				ldif_add_value( rec, lastTag, fullValue, hashField );
 				/* ldif_print_record( rec, stdout ); */
 				ldif_build_items( ldifFile, rec, cache );
 				ldif_clear_rec( rec );
+				g_free( fullValue );
 				g_free( lastTag );
 				mgu_free_list( listValue );
 				lastTag = NULL;
@@ -696,16 +733,20 @@ static void ldif_read_file( LdifFile *ldifFile, AddressCache *cache ) {
 					tagValue = ldif_get_tagvalue( line );
 					if( tagValue ) {
 						if( lastTag ) {
+							gchar *fullValue;
+
 							/* Save data */
 							fullValue = mgu_list_coalesce( listValue );
 							/* Base-64 encoded data */
-							/*
 							if( last64 ) {
-								ldif_dump_b64( fullValue );
+								gchar *decValue;
+								decValue = ldif_conv_base64( fullValue );
+								g_free( fullValue );
+								fullValue = decValue;
 							}
-							*/
 
 							ldif_add_value( rec, lastTag, fullValue, hashField );
+							g_free( fullValue );
 							g_free( lastTag );
 							mgu_free_list( listValue );
 							lastTag = NULL;
