@@ -26,52 +26,52 @@
 #include "stock_pixmap.h"
 #include "gtkutils.h"
 
-#include "pixmaps/address.xpm"
-#include "pixmaps/book.xpm"
-#include "pixmaps/category.xpm"
-#include "pixmaps/clip.xpm"
-#include "pixmaps/complete.xpm"
-#include "pixmaps/continue.xpm"
-#include "pixmaps/deleted.xpm"
-#include "pixmaps/dir-close.xpm"
-#include "pixmaps/dir-open.xpm"
-#include "pixmaps/dir-noselect.xpm"
-#include "pixmaps/error.xpm"
-#include "pixmaps/forwarded.xpm"
-#include "pixmaps/group.xpm"
-#include "pixmaps/interface.xpm"
-#include "pixmaps/jpilot.xpm"
-#include "pixmaps/ldap.xpm"
-#include "pixmaps/linewrap.xpm"
-#include "pixmaps/mark.xpm"
-#include "pixmaps/new.xpm"
-#include "pixmaps/replied.xpm"
-#include "pixmaps/unread.xpm"
-#include "pixmaps/vcard.xpm"
-#include "pixmaps/online.xpm"
-#include "pixmaps/offline.xpm"
-#include "pixmaps/mail.xpm"
-#include "pixmaps/stock_inbox.h"
-#include "pixmaps/stock_outbox.h"
-#include "pixmaps/stock_mail.h"
-#include "pixmaps/stock_attach.h"
-#include "pixmaps/stock_mail-compose.h"
-#include "pixmaps/stock_mail-compose_16.h"
-#include "pixmaps/stock_mail-forward.h"
-#include "pixmaps/stock_mail-receive.h"
-#include "pixmaps/stock_mail_receive_all.h"
-#include "pixmaps/stock_mail-reply.h"
-#include "pixmaps/stock_mail-reply-to-all.h"
-#include "pixmaps/stock_mail-send.h"
-#include "pixmaps/stock_mail_send_queue.h"
-#include "pixmaps/stock_insert-file.h"
-#include "pixmaps/stock_addressbook.h"
-#include "pixmaps/stock_delete.h"
-#include "pixmaps/stock_delete_16.h"
-#include "pixmaps/stock_spam.h"
-#include "pixmaps/stock_hand-signed.h"
-#include "pixmaps/stock_sylpheed.h"
-#include "pixmaps/sylpheed-logo.h"
+#include "icons/address.xpm"
+#include "icons/book.xpm"
+#include "icons/category.xpm"
+#include "icons/clip.xpm"
+#include "icons/complete.xpm"
+#include "icons/continue.xpm"
+#include "icons/deleted.xpm"
+#include "icons/dir-close.xpm"
+#include "icons/dir-open.xpm"
+#include "icons/dir-noselect.xpm"
+#include "icons/error.xpm"
+#include "icons/forwarded.xpm"
+#include "icons/group.xpm"
+#include "icons/interface.xpm"
+#include "icons/jpilot.xpm"
+#include "icons/ldap.xpm"
+#include "icons/linewrap.xpm"
+#include "icons/mark.xpm"
+#include "icons/new.xpm"
+#include "icons/replied.xpm"
+#include "icons/unread.xpm"
+#include "icons/vcard.xpm"
+#include "icons/online.xpm"
+#include "icons/offline.xpm"
+#include "icons/mail.xpm"
+#include "icons/stock_inbox.h"
+#include "icons/stock_outbox.h"
+#include "icons/stock_mail.h"
+#include "icons/stock_attach.h"
+#include "icons/stock_mail-compose.h"
+#include "icons/stock_mail-compose_16.h"
+#include "icons/stock_mail-forward.h"
+#include "icons/stock_mail-receive.h"
+#include "icons/stock_mail_receive_all.h"
+#include "icons/stock_mail-reply.h"
+#include "icons/stock_mail-reply-to-all.h"
+#include "icons/stock_mail-send.h"
+#include "icons/stock_mail_send_queue.h"
+#include "icons/stock_insert-file.h"
+#include "icons/stock_addressbook.h"
+#include "icons/stock_delete.h"
+#include "icons/stock_delete_16.h"
+#include "icons/stock_spam.h"
+#include "icons/stock_hand-signed.h"
+#include "icons/stock_sylpheed.h"
+#include "icons/sylpheed-logo.h"
 
 typedef struct _StockPixmapData	StockPixmapData;
 
@@ -138,30 +138,35 @@ static StockPixmapData pixmaps[] =
 	{NULL, NULL, NULL, NULL, sylpheed_logo, sizeof(sylpheed_logo), NULL, 0},
 };
 
-/* return newly constructed GtkPixmap from GdkPixmap */
-GtkWidget *stock_pixmap_widget(GtkWidget *window, StockPixmap icon)
-{
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
-
-	g_return_val_if_fail(window != NULL, NULL);
-	g_return_val_if_fail(icon >= 0 && icon < N_STOCK_PIXMAPS, NULL);
-
-	stock_pixmap_gdk(window, icon, &pixmap, &mask);
-	return gtk_pixmap_new(pixmap, mask);
-}
 
 GtkWidget *stock_pixbuf_widget(GtkWidget *window, StockPixmap icon)
 {
 	GdkPixbuf *pixbuf;
 
-	g_return_val_if_fail(window != NULL, NULL);
 	g_return_val_if_fail(icon >= 0 && icon < N_STOCK_PIXMAPS, NULL);
 
 	stock_pixbuf_gdk(window, icon, &pixbuf);
 	return gtk_image_new_from_pixbuf(pixbuf);
 }
 
+GtkWidget *stock_pixbuf_widget_scale(GtkWidget *window, StockPixmap icon,
+				     gint width, gint height)
+{
+	GdkPixbuf *pixbuf;
+	GdkPixbuf *scaled_pixbuf;
+	GtkWidget *image;
+
+	g_return_val_if_fail(icon >= 0 && icon < N_STOCK_PIXMAPS, NULL);
+
+	stock_pixbuf_gdk(window, icon, &pixbuf);
+	scaled_pixbuf = gdk_pixbuf_scale_simple(pixbuf, width, height,
+						GDK_INTERP_HYPER);
+	image = gtk_image_new_from_pixbuf(scaled_pixbuf);
+	g_object_unref(scaled_pixbuf);
+
+	return image;
+}
+
 /* create GdkPixmap if it has not created yet */
 gint stock_pixmap_gdk(GtkWidget *window, StockPixmap icon,
 		      GdkPixmap **pixmap, GdkBitmap **mask)
