@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -89,14 +89,12 @@ static GdkColor emphasis_color = {
 	(gushort)0xcfff
 };
 
-#if 0
 static GdkColor error_color = {
 	(gulong)0,
 	(gushort)0xefff,
 	(gushort)0,
 	(gushort)0
 };
-#endif
 
 #if USE_GPGME
 static GdkColor good_sig_color = {
@@ -147,6 +145,10 @@ static GdkCursor *regular_cursor = NULL;
 static void textview_add_part		(TextView	*textview,
 					 MimeInfo	*mimeinfo,
 					 FILE		*fp);
+#if USE_GPGME
+static void textview_add_sig_part	(TextView	*textview,
+					 MimeInfo	*mimeinfo);
+#endif
 static void textview_add_parts		(TextView	*textview,
 					 MimeInfo	*mimeinfo,
 					 FILE		*fp);
@@ -203,6 +205,9 @@ static void textview_popup_menu_activate_image_cb
 						(GtkMenuItem	*menuitem,
 						 gpointer	 data);
 
+static void textview_adj_value_changed		(GtkAdjustment	*adj,
+						 gpointer	 data);
+
 static void textview_smooth_scroll_do		(TextView	*textview,
 						 gfloat		 old_value,
 						 gfloat		 last_value,
@@ -249,7 +254,7 @@ TextView *textview_create(void)
 				       GTK_POLICY_AUTOMATIC,
 				       GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwin),
-					    GTK_SHADOW_IN);
+					    GTK_SHADOW_ETCHED_IN);
 	gtk_widget_set_size_request
 		(scrolledwin, prefs_common.mainview_width, -1);
 
@@ -282,6 +287,10 @@ TextView *textview_create(void)
 	g_signal_connect(G_OBJECT(text), "populate-popup",
 			 G_CALLBACK(textview_populate_popup), textview);
 
+	g_signal_connect(G_OBJECT(GTK_TEXT_VIEW(text)->vadjustment),
+			 "value-changed",
+			 G_CALLBACK(textview_adj_value_changed), textview);
+
 	gtk_widget_show(scrolledwin);
 
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -311,7 +320,7 @@ static void textview_create_tags(TextView *textview)
 	buffer = gtk_text_view_get_buffer(text);
 
 	gtk_text_buffer_create_tag(buffer, "header",
-				   "pixels-above-lines", 0,
+				   "pixels-above-lines", 1,
 				   "pixels-above-lines-set", TRUE,
 				   "pixels-below-lines", 0,
 				   "pixels-below-lines-set", TRUE,
@@ -321,6 +330,14 @@ static void textview_create_tags(TextView *textview)
 				   "weight", PANGO_WEIGHT_BOLD,
 				   NULL);
 
+	gtk_text_buffer_create_tag(buffer, "mimepart",
+				   "pixels-above-lines", 1,
+				   "pixels-above-lines-set", TRUE,
+				   "pixels-below-lines", 1,
+				   "pixels-below-lines-set", TRUE,
+				   "font-desc", font_desc,
+				   NULL);
+
 	textview->quote0_tag =
 		gtk_text_buffer_create_tag(buffer, "quote0",
 					   "foreground-gdk", &quote_colors[0],
@@ -346,6 +363,9 @@ static void textview_create_tags(TextView *textview)
 	gtk_text_buffer_create_tag(buffer, "emphasis",
 				   "foreground-gdk", &emphasis_color,
 				   NULL);
+	gtk_text_buffer_create_tag(buffer, "error",
+				   "foreground-gdk", &error_color,
+				   NULL);
 #if USE_GPGME
 	gtk_text_buffer_create_tag(buffer, "good-signature",
 				   "foreground-gdk", &good_sig_color,
@@ -430,7 +450,7 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 
 	buffer = gtk_text_view_get_buffer(text);
 
-	if ((fp = fopen(file, "rb")) == NULL) {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return;
 	}
@@ -441,6 +461,8 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 		charset = prefs_common.force_charset;
 	else if (mimeinfo->charset)
 		charset = mimeinfo->charset;
+	else if (prefs_common.default_encoding)
+		charset = prefs_common.default_encoding;
 
 	textview_set_font(textview, charset);
 	textview_clear(textview);
@@ -456,6 +478,11 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 	}
 
 	textview_add_parts(textview, mimeinfo, fp);
+#if USE_GPGME
+	if (textview->messageview->msginfo->encinfo &&
+	    textview->messageview->msginfo->encinfo->sigstatus)
+		textview_add_sig_part(textview, NULL);
+#endif
 
 	fclose(fp);
 
@@ -472,6 +499,10 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	const gchar *charset = NULL;
 	GPtrArray *headers = NULL;
 	gboolean is_rfc822_part = FALSE;
+	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	GtkTextMark *mark;
 
 	g_return_if_fail(mimeinfo != NULL);
 	g_return_if_fail(fp != NULL);
@@ -493,6 +524,8 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		charset = prefs_common.force_charset;
 	else if (mimeinfo->charset)
 		charset = mimeinfo->charset;
+	else if (prefs_common.default_encoding)
+		charset = prefs_common.default_encoding;
 
 	if (!boundary && mimeinfo->mime_type == MIME_TEXT) {
 		if (fseek(fp, mimeinfo->fpos, SEEK_SET) < 0)
@@ -540,18 +573,14 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	}
 
 	textview_set_font(textview, charset);
-
 	textview_clear(textview);
 
-	if (headers) {
-		GtkTextView *text = GTK_TEXT_VIEW(textview->text);
-		GtkTextBuffer *buffer;
-		GtkTextIter iter;
+	buffer = gtk_text_view_get_buffer(text);
 
+	if (headers) {
 		textview_show_header(textview, headers);
 		procheader_header_array_destroy(headers);
 
-		buffer = gtk_text_view_get_buffer(text);
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		textview->body_pos = gtk_text_iter_get_offset(&iter);
 		if (!mimeinfo->main)
@@ -562,6 +591,10 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		textview_add_parts(textview, mimeinfo, fp);
 	else
 		textview_write_body(textview, mimeinfo, fp, charset);
+
+	textview_set_position(textview, 0);
+	mark = gtk_text_buffer_get_insert(buffer);
+	gtk_text_view_scroll_mark_onscreen(text, mark);
 }
 
 static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
@@ -602,6 +635,8 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		charset = prefs_common.force_charset;
 	else if (mimeinfo->charset)
 		charset = mimeinfo->charset;
+	else if (prefs_common.default_encoding)
+		charset = prefs_common.default_encoding;
 
 	if (mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
 		headers = textview_scan_header(textview, fp, charset);
@@ -614,11 +649,12 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	}
 
 #if USE_GPGME
-	if (mimeinfo->sigstatus)
-		g_snprintf(buf, sizeof(buf), "\n[%s (%s)]\n",
-			   mimeinfo->content_type, mimeinfo->sigstatus);
-	else
+	if (mimeinfo->parent && mimeinfo->sigstatus) {
+		textview_add_sig_part(textview, mimeinfo);
+		return;
+	}
 #endif
+
 	if (mimeinfo->filename || mimeinfo->name)
 		g_snprintf(buf, sizeof(buf), "\n[%s  %s (%s)]\n",
 			   mimeinfo->filename ? mimeinfo->filename :
@@ -630,24 +666,10 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 			   mimeinfo->content_type,
 			   to_human_readable(mimeinfo->content_size));
 
-#if USE_GPGME
-	if (mimeinfo->sigstatus) {
-		const gchar *color;
-		if (!strcmp(mimeinfo->sigstatus, _("Good signature")))
-			color = "good-signature";
-		else if (!strcmp(mimeinfo->sigstatus, _("Valid signature (untrusted key)")))
-			color = "untrusted-signature";
-		else if (!strcmp(mimeinfo->sigstatus, _("BAD signature")))
-			color = "bad-signature";
-		else
-			color = "nocheck-signature";
-		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, buf, -1,
-							 color, NULL);
-	} else
-#endif
 	if (mimeinfo->mime_type != MIME_TEXT &&
 	    mimeinfo->mime_type != MIME_TEXT_HTML) {
-		gtk_text_buffer_insert(buffer, &iter, buf, -1);
+		gtk_text_buffer_insert_with_tags_by_name
+			(buffer, &iter, buf, -1, "mimepart", NULL);
 		if (mimeinfo->mime_type == MIME_IMAGE &&
 		    prefs_common.inline_image) {
 			GdkPixbuf *pixbuf;
@@ -704,13 +726,62 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		if (!mimeinfo->main &&
 		    mimeinfo->parent &&
 		    mimeinfo->parent->children != mimeinfo)
-			gtk_text_buffer_insert(buffer, &iter, buf, -1);
+			gtk_text_buffer_insert_with_tags_by_name
+				(buffer, &iter, buf, -1, "mimepart", NULL);
 		else
 			gtk_text_buffer_insert(buffer, &iter, "\n", 1);
 		textview_write_body(textview, mimeinfo, fp, charset);
 	}
 }
 
+#if USE_GPGME
+static void textview_add_sig_part(TextView *textview, MimeInfo *mimeinfo)
+{
+	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	gchar buf[BUFFSIZE];
+	const gchar *color;
+	const gchar *sigstatus;
+	const gchar *sigstatus_full;
+	const gchar *type;
+
+	if (mimeinfo) {
+		sigstatus = mimeinfo->sigstatus;
+		sigstatus_full = mimeinfo->sigstatus_full;
+		type = mimeinfo->content_type;
+	} else if (textview->messageview->msginfo->encinfo) {
+		sigstatus = textview->messageview->msginfo->encinfo->sigstatus;
+		sigstatus_full =
+			textview->messageview->msginfo->encinfo->sigstatus_full;
+		type = "signature";
+	} else
+		return;
+
+	if (!sigstatus)
+		return;
+
+	g_snprintf(buf, sizeof(buf), "\n[%s (%s)]\n", type, sigstatus);
+
+	if (!strcmp(sigstatus, _("Good signature")))
+		color = "good-signature";
+	else if (!strcmp(sigstatus, _("Valid signature (untrusted key)")))
+		color = "untrusted-signature";
+	else if (!strcmp(sigstatus, _("BAD signature")))
+		color = "bad-signature";
+	else
+		color = "nocheck-signature";
+
+	buffer = gtk_text_view_get_buffer(text);
+	gtk_text_buffer_get_end_iter(buffer, &iter);
+	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, buf, -1,
+						 color, "mimepart", NULL);
+	if (sigstatus_full)
+		gtk_text_buffer_insert_with_tags_by_name
+			(buffer, &iter, sigstatus_full, -1, "mimepart", NULL);
+}
+#endif
+
 static void textview_add_parts(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 {
 	gint level;
@@ -733,18 +804,22 @@ static void textview_add_parts(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	}
 }
 
-void textview_show_error(TextView *textview)
+static void textview_write_error(TextView *textview, const gchar *msg)
 {
 	GtkTextBuffer *buffer;
 	GtkTextIter iter;
 
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
+	gtk_text_buffer_get_end_iter(buffer, &iter);
+	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, msg, -1,
+						 "error", NULL);
+}
+
+void textview_show_error(TextView *textview)
+{
 	textview_set_font(textview, NULL);
 	textview_clear(textview);
-
-	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
-	gtk_text_buffer_get_start_iter(buffer, &iter);
-	gtk_text_buffer_insert(buffer, &iter,
-			       _("This message can't be displayed.\n"), -1);
+	textview_write_error(textview, _("This message can't be displayed.\n"));
 }
 
 static void textview_write_body(TextView *textview, MimeInfo *mimeinfo,
@@ -765,6 +840,11 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo,
 			while (fgets(buf, sizeof(buf), tmpfp) != NULL)
 				textview_write_line(textview, buf, conv);
 		fclose(tmpfp);
+	} else {
+		textview_write_error
+			(textview,
+			 _("The body text couldn't be displayed because "
+			   "writing to temporary file failed.\n"));
 	}
 
 	conv_code_converter_destroy(conv);
@@ -806,7 +886,7 @@ static gboolean get_uri_part(const gchar *start, const gchar *scanpos,
 
 	/* find end point of URI */
 	for (ep_ = scanpos; *ep_ != '\0'; ep_++) {
-		if (!isgraph(*(const guchar *)ep_) ||
+		if (!g_ascii_isgraph(*ep_) ||
 		    !isascii(*(const guchar *)ep_) ||
 		    strchr("()<>{}[]\"", *ep_))
 			break;
@@ -818,11 +898,9 @@ static gboolean get_uri_part(const gchar *start, const gchar *scanpos,
 	 * should pass some URI type to this function and decide on that whether
 	 * to perform punctuation stripping */
 
-#define IS_REAL_PUNCT(ch)	(ispunct(ch) && ((ch) != '/')) 
+#define IS_REAL_PUNCT(ch)	(g_ascii_ispunct(ch) && !strchr("/?=", ch)) 
 
-	for (; ep_ - 1 > scanpos + 1 &&
-	       IS_REAL_PUNCT(*(const guchar *)(ep_ - 1));
-	     ep_--)
+	for (; ep_ - 1 > scanpos + 1 && IS_REAL_PUNCT(*(ep_ - 1)); ep_--)
 		;
 
 #undef IS_REAL_PUNCT
@@ -837,16 +915,28 @@ static gchar *make_uri_string(const gchar *bp, const gchar *ep)
 	return g_strndup(bp, ep - bp);
 }
 
+static gchar *make_http_uri_string(const gchar *bp, const gchar *ep)
+{
+	gchar *tmp;
+	gchar *result;
+
+	tmp = g_strndup(bp, ep - bp);
+	result = g_strconcat("http://", tmp, NULL);
+	g_free(tmp);
+
+	return result;
+}
+
 /* valid mail address characters */
 #define IS_RFC822_CHAR(ch) \
 	(isascii(ch) && \
 	 (ch) > 32   && \
 	 (ch) != 127 && \
-	 !isspace(ch) && \
+	 !g_ascii_isspace(ch) && \
 	 !strchr("(),;<>\"", (ch)))
 
 /* alphabet and number within 7bit ASCII */
-#define IS_ASCII_ALNUM(ch)	(isascii(ch) && isalnum(ch))
+#define IS_ASCII_ALNUM(ch)	(isascii(ch) && g_ascii_isalnum(ch))
 
 /* get_email_part() - retrieves an email address. Returns TRUE if succesful */
 static gboolean get_email_part(const gchar *start, const gchar *scanpos,
@@ -955,7 +1045,7 @@ static void textview_make_clickable_parts(TextView *textview,
 		{"http://",  strcasestr, get_uri_part,   make_uri_string},
 		{"https://", strcasestr, get_uri_part,   make_uri_string},
 		{"ftp://",   strcasestr, get_uri_part,   make_uri_string},
-		{"www.",     strcasestr, get_uri_part,   make_uri_string},
+		{"www.",     strcasestr, get_uri_part,   make_http_uri_string},
 		{"mailto:",  strcasestr, get_uri_part,   make_uri_string},
 		{"@",        strcasestr, get_email_part, make_email_string}
 	};
@@ -1149,7 +1239,7 @@ static void textview_write_link(TextView *textview, const gchar *str,
 		gtk_text_buffer_insert(buffer, &iter, buf, bufp - buf);
 
 	r_uri = g_new(RemoteURI, 1);
-	r_uri->uri = g_strdup(uri);
+	r_uri->uri = g_strstrip(g_strdup(uri));
 	r_uri->filename = NULL;
 	r_uri->start = gtk_text_iter_get_offset(&iter);
 	gtk_text_buffer_insert_with_tags_by_name
@@ -1177,6 +1267,13 @@ void textview_clear(TextView *textview)
 
 void textview_destroy(TextView *textview)
 {
+	GtkTextBuffer *buffer;
+	GtkClipboard *clipboard;
+
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
+	clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+	gtk_text_buffer_remove_selection_clipboard(buffer, clipboard);
+
 	textview_uri_list_remove_all(textview->uri_list);
 	textview->uri_list = NULL;
 	g_free(textview);
@@ -1218,58 +1315,20 @@ void textview_set_position(TextView *textview, gint pos)
 static GPtrArray *textview_scan_header(TextView *textview, FILE *fp,
 				       const gchar *encoding)
 {
-	gchar buf[BUFFSIZE];
-	GPtrArray *headers, *sorted_headers;
-	GSList *disphdr_list;
-	Header *header;
-	gint i;
-
 	g_return_val_if_fail(fp != NULL, NULL);
 
 	if (textview->show_all_headers)
 		return procheader_get_header_array_asis(fp, encoding);
 
 	if (!prefs_common.display_header) {
+		gchar buf[BUFFSIZE];
+
 		while (fgets(buf, sizeof(buf), fp) != NULL)
 			if (buf[0] == '\r' || buf[0] == '\n') break;
 		return NULL;
 	}
 
-	headers = procheader_get_header_array_asis(fp, encoding);
-
-	sorted_headers = g_ptr_array_new();
-
-	for (disphdr_list = prefs_common.disphdr_list; disphdr_list != NULL;
-	     disphdr_list = disphdr_list->next) {
-		DisplayHeaderProp *dp =
-			(DisplayHeaderProp *)disphdr_list->data;
-
-		for (i = 0; i < headers->len; i++) {
-			header = g_ptr_array_index(headers, i);
-
-			if (!g_ascii_strcasecmp(header->name, dp->name)) {
-				if (dp->hidden)
-					procheader_header_free(header);
-				else
-					g_ptr_array_add(sorted_headers, header);
-
-				g_ptr_array_remove_index(headers, i);
-				i--;
-			}
-		}
-	}
-
-	if (prefs_common.show_other_header) {
-		for (i = 0; i < headers->len; i++) {
-			header = g_ptr_array_index(headers, i);
-			g_ptr_array_add(sorted_headers, header);
-		}
-		g_ptr_array_free(headers, TRUE);
-	} else
-		procheader_header_array_destroy(headers);
-
-
-	return sorted_headers;
+	return procheader_get_header_array_for_display(fp, encoding);
 }
 
 static void textview_show_header(TextView *textview, GPtrArray *headers)
@@ -1283,12 +1342,12 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 	g_return_if_fail(headers != NULL);
 
 	buffer = gtk_text_view_get_buffer(text);
+	gtk_text_buffer_get_end_iter(buffer, &iter);
 
 	for (i = 0; i < headers->len; i++) {
 		header = g_ptr_array_index(headers, i);
 		g_return_if_fail(header->name != NULL);
 
-		gtk_text_buffer_get_end_iter(buffer, &iter);
 		gtk_text_buffer_insert_with_tags_by_name
 			(buffer, &iter, header->name, -1,
 			 "header_title", "header", NULL);
@@ -1306,7 +1365,6 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 		    (!strncmp(header->name, "X-Mailer", 8) ||
 		     !strncmp(header->name, "X-Newsreader", 12)) &&
 		    strstr(header->body, "Sylpheed") != NULL) {
-			gtk_text_buffer_get_end_iter(buffer, &iter);
 			gtk_text_buffer_insert_with_tags_by_name
 				(buffer, &iter, header->body, -1,
 				 "header", "emphasis", NULL);
@@ -1317,7 +1375,7 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 			textview_make_clickable_parts
 				(textview, "header", NULL, header->body);
 		}
-		gtk_text_buffer_get_end_iter(buffer, &iter); //
+		gtk_text_buffer_get_end_iter(buffer, &iter);
 		gtk_text_buffer_insert_with_tags_by_name
 			(buffer, &iter, "\n", 1, "header", NULL);
 	}
@@ -1474,11 +1532,17 @@ static void textview_smooth_scroll_do(TextView *textview,
 
 	/* gdk_key_repeat_disable(); */
 
+	g_signal_handlers_block_by_func(vadj, textview_adj_value_changed,
+					textview);
+
 	for (i = step; i <= change_value; i += step) {
 		vadj->value = old_value + (up ? -i : i);
 		g_signal_emit_by_name(G_OBJECT(vadj), "value_changed", 0);
 	}
 
+	g_signal_handlers_unblock_by_func(vadj, textview_adj_value_changed,
+					  textview);
+
 	vadj->value = last_value;
 	g_signal_emit_by_name(G_OBJECT(vadj), "value_changed", 0);
 
@@ -1586,8 +1650,18 @@ static gboolean textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	case GDK_End:
 	case GDK_Control_L:
 	case GDK_Control_R:
+	case GDK_KP_Tab:
+	case GDK_KP_Home:
+	case GDK_KP_Left:
+	case GDK_KP_Up:
+	case GDK_KP_Right:
+	case GDK_KP_Down:
+	case GDK_KP_Page_Up:
+	case GDK_KP_Page_Down:
+	case GDK_KP_End:
 		break;
 	case GDK_space:
+	case GDK_KP_Space:
 		if (summaryview)
 			summary_pass_key_press_event(summaryview, event);
 		else
@@ -1600,11 +1674,13 @@ static gboolean textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		textview_scroll_page(textview, TRUE);
 		break;
 	case GDK_Return:
+	case GDK_KP_Enter:
 		textview_scroll_one_line
 			(textview, (event->state &
 				    (GDK_SHIFT_MASK|GDK_MOD1_MASK)) != 0);
 		break;
 	case GDK_Delete:
+	case GDK_KP_Delete:
 		if (summaryview)
 			summary_pass_key_press_event(summaryview, event);
 		break;
@@ -1761,11 +1837,7 @@ static void textview_show_uri(TextView *textview, GtkTextIter *start,
 	STATUSBAR_POP(textview);
 	uri = textview_get_uri(textview, start, end);
 	if (uri) {
-		gchar *trimmed_uri;
-
-		trimmed_uri = trim_string(uri->uri, 60);
-		STATUSBAR_PUSH(textview, trimmed_uri);
-		g_free(trimmed_uri);
+		STATUSBAR_PUSH(textview, uri->uri);
 	}
 }
 
@@ -2034,6 +2106,17 @@ static void textview_popup_menu_activate_image_cb(GtkMenuItem *menuitem,
 	g_free(src);
 }
 
+static void textview_adj_value_changed(GtkAdjustment *adj, gpointer data)
+{
+	TextView *textview = (TextView *)data;
+	GtkTextBuffer *buffer;
+
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
+	if (gtk_text_buffer_get_selection_bounds(buffer, NULL, NULL))
+		return;
+	gtk_text_view_place_cursor_onscreen(GTK_TEXT_VIEW(textview->text));
+}
+
 static gboolean textview_uri_security_check(TextView *textview, RemoteURI *uri)
 {
 	GtkTextBuffer *buffer;
@@ -2058,7 +2141,7 @@ static gboolean textview_uri_security_check(TextView *textview, RemoteURI *uri)
 
 		uri_path = get_uri_path(uri->uri);
 		visible_uri_path = get_uri_path(visible_str);
-		if (strcmp(uri_path, visible_uri_path) != 0)
+		if (path_cmp(uri_path, visible_uri_path) != 0)
 			retval = FALSE;
 	}
 
